/**********************************************************************
*  2006 Microchip Technology Inc.
*
* FileName:        main.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC24fj64ga002
* Compiler:        MPLAB C30 v2.02.00 or higher
*
* Running with Internal RC x 4PLL or 32Mhz -> 16MIPS
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author  Thomas Kibalo           Date   1/30/08   Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Using PICKIT2 with new version of MPLAB 8.01
* testing pic24 for robot sensor board servo timing generation using OC modules
* added capability to ping Parallel device--uses RB5 and IC1--works!
* 11/23 issue with inches/measurement not being long CPU crashed--raised exception
* 11/24 made measurement and inches into longs excpetion disappeared
* 11/24 added ping1 for IC1
* 11/24 added servo initialization and servo functions servo on/off servo set
*11/24 copied RadioShack PICKIT_PIC24Fver3 and made BlueRoguever0 
*05/8/10 coverted to ping just reading sensor for Experimenter
left IC1 RP5 pin 3 along with RB5 I/O for ping snesor
assigend OC1 RP12 pin 7 for servo #1
assigend OC2 RP13 pin 8 for servo #2
*5/8/10 converted ping to autoscan by adding servo control in conjuction with ping
*5/9/10 coverted to scan --added buttons
*7/13/10 begin conversion of file  to RGB experiments
RED	 pin 1 expansion bus RP2 RB2 OC1
BLUE pin 3 expansion bus RP3 RB3 OC2
GREEN pin 7 expansion bus RP12 RB12 OC3 
*7/15/10 PPS configurATIONS for RGB
*7/15/10 conversion of RGB to Motor Demo
pin1  OC2 RB2/RP2  MOTOR2
pin2 enable2 RB3
pin8  OC1 RB13/RP13 MOTOR1
pin10 analog RB15
pin9 enable1 RB14
need to fix PPS for OC
?add duty cycle display?

***************************************************/
// Include Header File
//***************************************************/

#include "p24fj64ga002.h"
#include "lcd.h"
#include "adc.h"
#include "delay.h"
#include "buttons.h"

#define enable2 LATBbits.LATB3
#define enable1  LATBbits.LATB14

#define OC2_ON  OC2CONbits.OCM =6

#define OC1_ON  OC1CONbits.OCM =6

#define OC2_OFF OC2CONbits.OCM =0
#define OC1_OFF OC1CONbits.OCM =0

char bcd1000=0;
char bcd100 =0;
char bcdtens =0;
char bcdunits =0; 
//int result;
unsigned int setting =0;
unsigned int adcvalue =0;

const char banner[] = "Motor demo";
const char mmotor1[] = "Motor1 sel";
const char mmotor2[] = "Motor2 sel";


;

int selMotor =0;
void binary_to_ASCIIconvert( int n) {
bcd100 =0x00;
bcdtens =0x00;
bcdunits =0x00;
bcd1000 =0x00;
while (n>=1000) {
n = n-1000;
bcd1000 =bcd1000+1;
}
while (n>=100) {
n = n-100;
bcd100 =bcd100+1;
}
while (n>=10) {
n = n-10;
bcdtens =bcdtens+1;
}
while (n>0) {
n = n-1;
bcdunits =bcdunits+1;
}

bcd1000 =bcd1000+0x30;
bcd100 =bcd100+0x30;
bcdtens =bcdtens+0x30;
bcdunits =bcdunits+0x30;
}



/*********************************************************************************
* servo functions ****************************************************************
*********************************************************************************/

void Motor_init(void) { //initializes servo on OC1, OC2, OC3, OC4  uses Timer2
	PR2 =0x3ff;    //20 msec @ 16 MIP
	TMR2 =0x0000;
	T2CON = 0x0000;  // no 32 bit mod presecal no prescale fosc/2 clock ( 16MIPS-> 2 MIPS or 500 nanosec
	T2CONbits.TCKPS= 0x01;  //prescaler 1:8
	T2CONbits.TON =0; //turn it off

	OC1RS = 0;	//zero pwm
	OC1CON = 0x0006;  // TMR2, PWM
	OC2RS =0;	// zero pwm
	OC2CON = 0x0006;  // TMR2, PWM


	OC1_ON;
	OC2_ON;


	T2CONbits.TON =1; //turn it on
	
	}

	



void set_Motor1( int set) {
	OC1RS =set;
	
}
void Motor1_off( void) {
	OC1_OFF;
}
void Motor2_off( void) {
	OC2_OFF	;
}
void set_Motor2( int set) {
	OC2RS =set;
}
	

	
//***************************************************/
int main (void)
{
	CLKDIVbits.RCDIV = 0; //no clk div stay at 16MIPS

__builtin_write_OSCCONL(OSCCON & 0xbf) ; //clear bit 6
//pin1  OC2 RB2/RP2
//pin2 enable2 RB3
//pin8  OC1 RB13/RP13
//pin10 analog RB15
//pin9 enable1 RB14

//output
	RPOR1bits.RP2R =19; // OC2 becomes RP2 PIN 1 I/O exp MOTOR2
	RPOR6bits.RP13R =18; // OC1 becomes RP13  pin 8 I/O exp MOTOR1

__builtin_write_OSCCONL(OSCCON | 0x40) ; //set bit 6

TRISBbits.TRISB2 =0;
TRISBbits.TRISB3 =0;
TRISBbits.TRISB15 =0;
TRISBbits.TRISB13 =0;
TRISBbits.TRISB14 =0;

enable1 =0;
enable2 =0;

LCD_Initialize();
initADC( pin10); //initialize pin9 for analog
clear_display();
write_string_LCD( (char*) banner);
BtnInit();						//initialize button operation

int button=0;					// create a button variable for use in loop



	Motor_init();    ///initializes OC1 and OC2 for center positions of servos
	

delay() ;  //wiat until all servos are centered


while(1) {
	if (selMotor >0 ) {
	   //	delay();
	   	adcvalue = readADC (pin10);
// ADC value with be between 0 -3ff count =0 to 1024


setting = adcvalue ;  //multiply x2 --now can be used for servo setting



  switch (selMotor) {
	  
	  case 1:
	  
	  set_Motor1(setting);
	  
	 
	  break;
	  case 2:
	  
	  set_Motor2(setting);
	  
	  
	  break;
	  
	  
	  
	  default:
	  
	  selMotor=0;
	set_Motor1(0);
 	set_Motor2(0);
 	clear_display();
write_string_LCD( (char*) banner);

enable2=0;
	  enable1 =0;
	  break;	  
	  
	  
	  
	 }	
	  	
	   	
  
  	
  		}		 		


 button = process_buttons();	// process buttons and report back through button variable
 								// if button is equal to zero no pushbuttons were depressed
 
 if ( button==1) {				// if pushbutton SW1 was pressed the display its message
	
				position_cursor(row2);
   			
   				 write_string_LCD( (char*)mmotor1); 
   				//set_servo1(setting);
   				enable2=0;
	 			 enable1 =1;
   				selMotor =1;
   				
     		
   				//delay(); 
   				
 }  		
  if ( button==2) {				//if pushbutton SW2 was pressed the display its message
	
				position_cursor(row2);
   			 write_string_LCD( (char*)mmotor2); 
   			//	set_servo2(setting);
   			enable1 =0;
   			enable2=1;
	  
   					selMotor =2;
   			
   			
   			 
   				//delay(); 
  }
   if ( button==3) {			//if pustbutton SW3 was pressed the display its message
		
			//do nothing
   			
   			
   			//	delay(); 
  }
   	if ( button==4) {			//if pushbutton SW4 was pressed the display its message
		
			//do nothing
   		//	delay(); 
   


}		

}
}
//end of code
