/*****************************************************************************
 *  Buttons
 *****************************************************************************

 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * thk 11/21/09  derived form microchip soruce
 *****************************************************************************/

/*****************************************************************************
 * Buttons IOs PORT definitions for different versions
 * of Explorer 16 Development Board.
 *****************************************************************************/

#define BUTTON1 PORTAbits.RA0    // toggle between set mode and clock mode
#define BUTTON2 PORTAbits.RA1	 // incr when in setup
#define BUTTON3 PORTAbits.RA2	 //decr when in setup
#define BUTTON4 PORTAbits.RA3   // select next colkc/date field for change if in setup
#define BUTTON5 PORTBbits.RB12  //toggle between back light on /off

#define BLACKLIGHT LATBbits.LATB13

int  process_buttons(void);

/*****************************************************************************
 * Function: BtnInit
 *
 * Preconditon: None.
 *
 * Overview: Setup debounce.
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/
void BtnInit(void);

/*****************************************************************************
 * Function: BtnProcessEvents
 *
 * Preconditon: None.
 *
 * Overview: Must be called periodically to proccess buttons.
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/
void BtnProcessEvents(void);
void toggle_BL(void);
void QBtnProcessEvents(void);

/*****************************************************************************
 * Structure: BUTTONS _button_press
 *
 * Overview: the structure provides a n access to button bit indicators.
 *
 *****************************************************************************/
typedef struct tagButton {
	unsigned b1:1;
	unsigned b2:1;
	unsigned b3:1;
	unsigned b4:1;
	unsigned b5:1;
}BUTTON;
extern BUTTON _button_press;

/*****************************************************************************
 * Debounce button counters
 *
 *****************************************************************************/
#define	BUTTON_MAX_DEBOUCE 	2
extern unsigned char _b1_cnt;
extern unsigned char _b2_cnt;
extern unsigned char _b3_cnt;
extern unsigned char _b4_cnt;
extern unsigned char _b5_cnt;
/*****************************************************************************
 * Function: BtnIsPressed
 *
 * Preconditon: None.
 *
 * Overview: Macro detects if button is pressed
 *
 * Input: None.
 *
 * Output: Macro returns zero if button is not pressed.
 *
 *****************************************************************************/
#define BtnIsPressed(__btn)	_button_press.b##__btn

/*****************************************************************************
 * Function: BtnIsNotPressed
 *
 * Preconditon: None.
 *
 * Overview: Macro detects if button is not pressed
 *
 * Input: None.
 *
 * Output: Macro returns zero if button is pressed.
 *
 *****************************************************************************/
#define BtnIsNotPressed(__btn)	!_button_press.b##__btn

/*****************************************************************************
 * EOF
 *****************************************************************************/
