/**********************************************************************
*  2006 Microchip Technology Inc.
*
* FileName:        main.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC24fj64ga002
* Compiler:        MPLAB C30 v2.02.00 or higher
*
* Running with Internal RC x 4PLL or 32Mhz -> 16MIPS
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author  Thomas Kibalo           Date   1/30/08   Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Using PICKIT2 with new version of MPLAB 8.01
* testing pic24 for robot sensor board servo timing generation using OC modules
* added capability to ping Parallel device--uses RB5 and IC1--works!
* 11/23 issue with inches/measurement not being long CPU crashed--raised exception
* 11/24 made measurement and inches into longs excpetion disappeared
* 11/24 added ping1 for IC1
* 11/24 added servo initialization and servo functions servo on/off servo set
*11/24 copied RadioShack PICKIT_PIC24Fver3 and made BlueRoguever0 
*05/8/10 coverted to ping just reading sensor for Experimenter
left IC1 RP5 pin 3 along with RB5 I/O for ping snesor
assigend OC1 RP12 pin 7 for servo #1
assigend OC2 RP13 pin 8 for servo #2
*5/8/10 converted ping to autoscan by adding servo control in conjuction with ping
*5/9/10 coverted to scan --added buttons
*7/15/10 reassigned OC as follows
pin 3 OC1  RP5	Servo 1
pin 7 OC2  RP12 Servo 2
pin 8 OC3  RP13	Servo 3
pin 9 OC4  RP14	Servo 4
pin 10 analog

***************************************************/
// Include Header File
//***************************************************/

#include "p24fj64ga002.h"
#include "lcd.h"
#include "adc.h"
#include "delay.h"
#include "buttons.h"

//***************************************************/
//
//***************************************************/
// Define Constants Variables
//***************************************************/
#define center 0x0BB8 //1.5msec
#define left   0x07D0 // 1 msec
#define right  0x0FA0 //2 msec

//PING interface uses IC1 

#define IC1_RISE IC1CONbits.ICM= 3
#define IC1_FALL IC1CONbits.ICM=2
#define IC1_OFF  IC1CONbits.ICM=0
#define RB5_OUT TRISBbits.TRISB5=0;
#define RB5_IN  TRISBbits.TRISB5=1;
#define out_Pulse_high LATBbits.LATB5=1;
#define out_Pulse_low  LATBbits.LATB5 =0;

#define OC2_ON  OC2CONbits.OCM =6
#define OC2_OFF OC2CONbits.OCM =0
#define OC1_ON  OC1CONbits.OCM =6
#define OC1_OFF OC1CONbits.OCM =0
#define OC3_ON  OC3CONbits.OCM =6
#define OC3_OFF OC3CONbits.OCM =0
#define OC4_ON  OC4CONbits.OCM =6
#define OC4_OFF OC4CONbits.OCM =0
#define LF	0x0A
#define CR	0x0D
#define SPACE 0x20
#define iend 500
#define jend 1000

static unsigned char capture_state =0;
static int high_time =0;
static int low_time=0;
static long measure =0;
static long inches;
char bcd1000=0;
char bcd100 =0;
char bcdtens =0;
char bcdunits =0; 
//int result;
const char banner[] = "Servo demo";
const char mservo1[] = "1 selected";
const char mservo2[] = "2 selected";
const char mservo3[] = "3 selected";
const char mservo4[] = "4 selected";

int servo1pos = center;
int servo2pos = center;
int servo3pos = center;
int servo4pos = center;
int selservo =0;
void binary_to_ASCIIconvert( int n) {
bcd100 =0x00;
bcdtens =0x00;
bcdunits =0x00;
bcd1000 =0x00;
while (n>=1000) {
n = n-1000;
bcd1000 =bcd1000+1;
}
while (n>=100) {
n = n-100;
bcd100 =bcd100+1;
}
while (n>=10) {
n = n-10;
bcdtens =bcdtens+1;
}
while (n>0) {
n = n-1;
bcdunits =bcdunits+1;
}

bcd1000 =bcd1000+0x30;
bcd100 =bcd100+0x30;
bcdtens =bcdtens+0x30;
bcdunits =bcdunits+0x30;
}



/*********************************************************************************
* servo functions ****************************************************************
*********************************************************************************/

void servo_init(void) { //initializes servo on OC1 and OC2 to center positions uses Timer2
	PR2 =0x9C40;    //20 msec @ 16 MIP
	TMR2 =0x0000;
	T2CON = 0x0000;  // no 32 bit mod presecal no prescale fosc/2 clock ( 16MIPS-> 2 MIPS or 500 nanosec
	T2CONbits.TCKPS= 0x01;  //prescaler 1:8
	T2CONbits.TON =0; //turn it off
	OC1RS = center;	//1.5msec
	OC1CON = 0x0006;  // TMR2, PWM
	OC2RS =center;	// 1.5 msec
	OC2CON = 0x0006;  // TMR2, PWM
	OC3RS =center;	// 1.5 msec
	OC3CON = 0x0006;  // TMR2, PWM
	OC4RS =center;	// 1.5 msec
	OC4CON = 0x0006;  // TMR2, PWM
	OC1_ON;
	OC2_ON;
	OC3_ON;
	OC4_ON;
	T2CONbits.TON =1; //turn it on
	
	}

void servo1_off( void) {
	OC1_OFF;
}
void servo1_on( void) {
	OC1_ON;
}
void servo2_off( void) {
	OC2_OFF	;
}
void servo2_on( void) {
	OC2_ON;
}		

void set_servo3( int set) {
	OC3RS =set;
}
void set_servo4( int set) {
	OC4RS =set;
}
void servo3_off( void) {
	OC3_OFF;
}
void servo3_on( void) {
	OC3_ON;
}
void servo4_off( void) {
	OC4_OFF	;
}
void servo4_on( void) {
	OC4_ON;
}		

void set_servo1( int set) {
	OC1RS =set;
}
void set_servo2( int set) {
	OC2RS =set;
}
//*******************************************************
void display(void) { //displays measurement on display
	measure = (long) (low_time- high_time);
	measure =measure/4 ;  //remove dual path first 2 divde and adjust cout second divdie by 2
   inches =measure/75 ;
   //convert inches to ascii;
     binary_to_ASCIIconvert((int)inches);
   
position_cursor(row2+6);
write_character_LCD(bcd100);
write_character_LCD(bcdtens);
write_character_LCD(bcdunits);	
	
}
//**********************************************************
void blink_led (void) { //blinks led for update and delays
	PORTBbits.RB15 =1;  //pin 10 of expansion bus -LED
	delay();
	delay();
	delay();
	PORTBbits.RB15 =0; //pin 10 of expansion bus -LED
	delay();

	
	
}		
/*************** PING ROUNTINE ***********************************
// interrupt service routine for ping1 IC1
*****************************************************************/
void __attribute__((interrupt, no_auto_psv)) _IC1Interrupt(void){
//capture the echo return from PING using interrupt state machine
	
	switch (capture_state)
	{
	case 0:  //received leading edge
		capture_state =1;
		high_time =IC1BUF;	//retrieve capture time to rising edge of echo
		IC1_FALL;			//set IC1 to capture falling edge of echo
		break;
	case 1: // received falling edge
		low_time=IC1BUF	;	//retreive capture time to falling edge of echo
		IC1_OFF;			//turn off IC1
		_IC1IE=0;			//disable interrupt
		capture_state=2;	//set state to completion
	
		break;
	default:
		capture_state =0;	//default 
		IC1_OFF;
			_IC1IE=0;
		break;
	}
				
	_IC1IF =0;		//on exit clear IC1 interrupt flag
		
		
	}
// ping1 service for RADIO SHACK PING device --used interrupt state machine and IC1
	
void ping1(void) { //pings on IC1---only one ICX is enabled at at time
	//all pings use global int low/high and global long uses Timer3
		IC1CON =0x0000;		//enable IC1
		PR3 =0xffff;		//initialize timer3
 		T3CON = 0x0010 ;    //1:8 prescale
		TMR3 = 0x0000;  //clear timer3
		_IC1IE =0;
		IC1_OFF;		//turn off IC1
	   _IC1IF =0;
		capture_state =0; //initialize state machine for interrupt
		RB5_OUT;		//set RB5 as output
		out_Pulse_low;	//RB5 pulse out for trigger
		delay_5us();	//5 usec pulse
		out_Pulse_high;
		delay_5us();
		out_Pulse_low;
		delay_5us();
		RB5_IN;			// set RB5 as input
		delay_100us();	//hold off
		delay_100us();
		delay_100us();
		delay_100us();
		T3CONbits.TON =1; //turn on timer
		IC1_RISE;		//capture rise
		_IC1IF =0;
		_IC1IE =1;		//enable interrupt
	while (capture_state !=2); //wait for state machine shows completion

 	T3CONbits.TON =0;  //turn off timer
	
		
}		
	

	
//***************************************************/
int main (void)
{
	CLKDIVbits.RCDIV = 0; //no clk div stay at 16MIPS

__builtin_write_OSCCONL(OSCCON & 0xbf) ; //clear bit 6

//output

//	RPOR6bits.RP12R =18; // OC1 becomes RP12  pin 7
//	RPOR6bits.RP13R =19; // OC2 becomes RP13 pin 8
//	RPOR2bits.RP5R =20;		//OC3 becomes RP5 pin3
//	RPOR7bits.RP15R =21;		//OC4 becomes RP15 pin10
//	pin 3 OC1  RP5
//	pin 7 OC2  RP12
//	pin 8 OC3  RP13
//	pin 9 OC4  RP14
//	pin 10 analog

	RPOR2bits.RP5R =18 ;  //OC1 becomes RP5 pin 3 Servo 1
	RPOR6bits.RP12R =19; // OC2 becomes RP12  pin 7 Servo 2
	RPOR6bits.RP13R =20; // OC3 becomes RP13 pin 8 Servo 3
	RPOR7bits.RP14R =21;  //OC4 becomes RP14 pin 9 Servo4
__builtin_write_OSCCONL(OSCCON | 0x40) ; //set bit 6

TRISBbits.TRISB12 =0;
TRISBbits.TRISB13 =0;
TRISBbits.TRISB5 =0;
TRISBbits.TRISB14 =0;

_ODB12=1;	//set servos out to open drain---need +5V out through LM324 op amp
 _ODB13=1; // pin 8 is OC1 RP13  , pin 7 is OC2 RP12
 _ODB5=1;  // pin 3 OC3 RP5
   _ODB14=1; //pin 9 OC4 RP15
// LED Peripheral Initialisation

LCD_Initialize();
initADC( pin10); //initialize pin10 for analog
clear_display();
write_string_LCD( (char*) banner);
BtnInit();						//initialize button operation

int button=0;					// create a button variable for use in loop
int setting =0;
int adcvalue =0;



	servo_init();    ///initializes OC1 and OC2 for center positions of servos
	

delay() ;  //wiat until all servos are centered
servo1_off();
servo2_off();
servo3_off();
servo4_off();

while(1) {
	if (selservo >0 ) {
	   //	delay();
	   	adcvalue = readADC (pin10);
// ADC value with be between 0 -3ff count =0 to 1024
//	   center 0x0BB8 //1.5msec count =3000
//       left   0x07D0 // 1 msec count =2000
//       right  0x0FA0 //2 msec count =4000


if (adcvalue >=24) adcvalue = adcvalue -24 ; //normalize to 0-1000
setting = adcvalue*2 +2000 ;  //multiply x2 --now can be used for servo setting
//position_cursor(row3);
//binary_to_ASCIIconvert(setting);
//	write_character_LCD(bcd1000);
//	write_character_LCD(bcd100);
//	write_character_LCD(bcdtens);
//	write_character_LCD(bcdunits);
  switch (selservo) {
	  
	  case 1:
	  set_servo1(setting);
	 
	  break;
	  case 2:
	  set_servo2(setting);
	 
	  
	  break;
	  
	  case 3:
	  set_servo3(setting);
	 
	  
	  break;
	  case 4:
	  set_servo4(setting);
	 
	  
	  break;
	  
	  default:
	  
	  selservo=0;
	  servo1_off();
servo2_off();
servo3_off();
servo4_off();
write_string_LCD( (char*) banner);

	  break;	  
	  
	  
	  
	 }	
	  	
	   	
  
  	
  		}		 		


 button = process_buttons();	// process buttons and report back through button variable
 								// if button is equal to zero no pushbuttons were depressed
 
 if ( button==1) {				// if pushbutton SW1 was pressed the display its message
	
				position_cursor(row2);
   				write_string_LCD((char*)mservo1);
   				//set_servo1(setting);
   				servo1pos =center;
   				set_servo1(servo1pos);
   				selservo =1;
   				servo1_on(); 
     			servo2_off();
				servo3_off();
				servo4_off();
   				//delay(); 
   				
 }  		
  if ( button==2) {				//if pushbutton SW2 was pressed the display its message
	
				position_cursor(row2);
   				write_string_LCD((char*)mservo2); 
   			//	set_servo2(setting);
   			servo2pos =center;
   					set_servo2(servo2pos);
   					selservo =2;
   				servo1_off();
   				servo2_on();
   				servo3_off();
				servo4_off(); 
   				//delay(); 
  }
   if ( button==3) {			//if pustbutton SW3 was pressed the display its message
		
				position_cursor(row2);
   				write_string_LCD((char*)mservo3);
   				servo1_off(); 
   				servo3pos =center;
   			//	set_servo3(setting);
   				set_servo3(servo3pos);
   				selservo =3;
   				servo2_off();
   				servo3_on();
   					servo4_off();  
   			//	delay(); 
  }
   	if ( button==4) {			//if pushbutton SW4 was pressed the display its message
		
				position_cursor(row2);
   				write_string_LCD((char*)mservo4); 
   				//set_servo4(setting);
   					servo4pos =center;
   					set_servo4(servo4pos);
   						selservo =4;
   				servo1_off();
   				servo2_off();
				servo3_off();
   				servo4_on();
   		//	delay(); 
   

//servo1_off();
//servo2_off();
//servo3_off();
//servo4_off();
	
}		

}
}
//end of code
